/**
 *
 *    Copyright (C) 2017 J. Robert (FAU Erlangen-Nuernberg)
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *    \brief This file contains the implementions for required noise functions.
 *
 */




#include <math.h>
#include <stdlib.h>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "ieee_lpwa_noise.h"




void gaussian_noise(double *data_re_p, double *data_im_p, const long int num) {
    long int i;

    for (i = 0; i < num; i++) {
        // Get two random numbers u1 and u2 in [0,1)
        const double u1 = rand_uniform();
        const double u2 = rand_uniform();

        // Use the Box Muller method to obtain complex Gaussian noise
        const double log_term = sqrt(-log(u1));
        data_re_p[i] = log_term * cos((2.0 * M_PI) * u2);
        data_im_p[i] = log_term * sin((2.0 * M_PI) * u2);
    }

    return;
}


void gaussian_noise_real(double *data_p, const long int num) {
    long int i;

    for (i = 0; i < num; i++) {
        // Get two random numbers u1 and u2 in [0,1)
        const double u1 = rand_uniform();
        const double u2 = rand_uniform();

        // Use the Box Muller method to obtain complex Gaussian noise
        const double log_term = sqrt(-2.0 * log(u1));
        data_p[i] = log_term * cos((2.0 * M_PI) * u2);
    }

    return;
}

long int noise_floor(double **data_re_pp, double **data_im_pp,
        const unsigned int num, double variance_lin) {

    // Allocate memory
    *data_re_pp = (double*)malloc(sizeof(double) * num);
    *data_im_pp = (double*)malloc(sizeof(double) * num);

    if (*data_re_pp == NULL || *data_im_pp == NULL)
        return -1;

    // Fill with complex Gaussian noise with variance one
    gaussian_noise(*data_re_pp, *data_im_pp, num);

    // Scale noise if variance != 1
    if (variance_lin != 1.0) {
        const double sigma = sqrt(variance_lin);
        long int i;
        for (i = 0; i < num; i++) {
            (*data_re_pp)[i] *= sigma;
            (*data_im_pp)[i] *= sigma;
        }
    }
    return num;
}

