/**
 *
 *    Copyright (C) 2017 J. Robert (FAU Erlangen-Nuernberg)
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *    \brief This file is an interface to export the required routines of the random generators
 *      defined in mt19937ar.c
 *
 */


#ifndef RAND_GENERATOR_H_ 
#define RAND_GENERATOR_H_ 
/*!
 * \brief init random generator with seed
 * 
 * \param    seed   Seed for random generator
 */
void init_genrand(unsigned long seed);

/*!
 * \brief Generates an integer random number on the invterval [0,0xffffffff]
 * 
 * \return Random number in the interval [0,0xffffffff] with uniform distribution
 */
unsigned long genrand_int32(void);

/*!
 * \brief Generates a daoubler on the interval [0,1) 
 * 
 * \return Random number in the interval [0,1) with uniform distribution
 */
double genrand_real2(void);


#endif /* RAND_GENERATOR_H_ */
